from typing import Any

class Configurations:
    data_path: Any = ...
    jisyo_hepburn_hira: str = ...
    jisyo_passport_hira: str = ...
    jisyo_kunrei_hira: str = ...
    jisyo_itaiji: str = ...
    jisyo_kanwa: str = ...
    jisyo_hepburn: str = ...
    jisyo_passport: str = ...
    jisyo_kunrei: str = ...
    jisyo_halfkana: str = ...
    def dictpath(self, dbfile: str) -> Any: ...

class Ch:
    space: int = ...
    at_mark: int = ...
    alphabet_A: int = ...
    alphabet_Z: int = ...
    square_bra: int = ...
    back_quote: int = ...
    alphabet_a: int = ...
    alphabet_z: int = ...
    bracket_bra: int = ...
    tilda: int = ...
    delete: int = ...
    ideographic_space: int = ...
    postal_mark_face: int = ...
    wavy_dash: int = ...
    ideographic_half_fill_space: int = ...
    greece_Alpha: int = ...
    greece_Rho: int = ...
    greece_Sigma: int = ...
    greece_Omega: int = ...
    greece_alpha: int = ...
    greece_omega: int = ...
    cyrillic_A: int = ...
    cyrillic_E: int = ...
    cyrillic_e: int = ...
    cyrillic_ya: int = ...
    zenkaku_exc_mark: int = ...
    zenkaku_slash_mark: int = ...
    zenkaku_number_zero: int = ...
    zenkaku_number_nine: int = ...
    zenkaku_A: int = ...
    zenkaku_a: int = ...
    endmark: str = ...

class Convert_Tables:
    symbol_table_1: Any = ...
    symbol_table_2: Any = ...
    symbol_table_3: Any = ...
    symbol_table_4: Any = ...
    symbol_table_5: Any = ...
    cyrillic_table: Any = ...
    alpha_table_1: Any = ...
    alpha_table_2: Any = ...
    alpha_table_3: Any = ...
