from .properties import (
    Ch as Ch,
    Configurations as Configurations,
    Convert_Tables as Convert_Tables,
)
from typing import Any, Dict

class IConv:
    def __init__(self) -> None: ...
    def convert(self, otext: str, hira: str) -> Dict[str, str]: ...

class H2:
    convert: Any = ...
    def __init__(self, mode: Any, method: str = ...) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_K(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class K2:
    convert: Any = ...
    def __init__(self, mode: Any, method: str = ...) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_h(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class Jisyo:
    def __init__(self, dictname: Any) -> None: ...
    def haskey(self, key: Any): ...
    def lookup(self, key: Any): ...
    def maxkeylen(self): ...

class Sym2:
    convert: Any = ...
    def __init__(self, mode: Any) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_a(self, text: Any): ...
    def convert_noop(self, text: Any): ...

class A2:
    convert: Any = ...
    def __init__(self, mode: Any) -> None: ...
    def isRegion(self, char: Any): ...
    def convert_E(self, text: Any): ...
    def convert_noop(self, text: Any): ...
