from .properties import Configurations as Configurations
from typing import Any, Tuple

class JConv:
    def __init__(self) -> None: ...
    def isRegion(self, c: str) -> Any: ...
    def convert(self, itext: str) -> Tuple[str, int]: ...

class Itaiji:
    __dict__: Any = ...
    def __new__(cls, *p: Any, **k: Any): ...
    def __init__(self) -> None: ...
    def haskey(self, c: Any): ...
    def convert(self, text: str) -> str: ...

class Kanwa:
    __dict__: Any = ...
    def __new__(cls, *p: Any, **k: Any): ...
    def __init__(self) -> None: ...
    def load(self, char: str) -> Any: ...
